<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
 "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head profile="http://www.w3.org/2005/10/profile">
  <meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
  <script type="text/javascript">
    {% if status == "ok" %}
    parent.webmail.upload_success("{{ fname }}", "{{ tmpname }}");
    {% else %}
    parent.webmail.upload_error("{{ error }}");
    {% endif %}
  </script>
</head>
<body></body>
</html>
