/* MEDIA */

@media (min-width: 768px) {
    .main {
        padding-left: 0;
        padding-right: 0;
    }
}

@media (max-width: 1198px) {
    .container-fluid {
        padding: 0;
    }
  
    .display-up-1198-webmail {
        display: none;
    }
    .display-less-1198{
        display: inherit;
    }

}

/* GENERAL */

.main {
    position: absolute;
    top: 51px;
    left: 0;
    right: 0;
    bottom: 0;
    overflow-x: hidden;
}

#listing {
    position: absolute;
    top: 75px;
    bottom: 0;
    left: 0;
    right: 0;
    overflow-y: auto;
    overflow-x: hidden;
    padding-left: 15px;
    padding-right: 15px;
}

#folders {
    top: 73px;
    left: 15px;
    right: 15px;
    overflow-x: auto;
    overflow-y: auto;
}

/* .sidebar .ui-resizable-handle { */
/*     background-position: center center; */
/*     background-repeat: no-repeat; */
/* } */

/* .sidebar .ui-resizable-n { */
/*     top: 0px; */
/* } */

/* .sidebar .ui-resizable-s { */
/*     bottom: 0px; */
/* } */

/* .sidebar .ui-resizable-e { */
/*     right: 0px; */
/*     background-image: url("../../pics/grippy.png"); */
/*     width: 8px; */
/* } */

/* .sidebar .ui-resizable-w { */
/*     left: 0px; */
/*     background-image: url("../../pics/grippy.png"); */
/*     width: 8px; */
/* } */

.nav-sidebar li > a {
    padding-top: 7px;
    padding-bottom: 7px;
}

.nav-sidebar li li { 
    padding-left: 10px;
}

#mboxes_container a[name=loadfolder] {
    white-space: nowrap;
    overflow: auto;
    text-overflow: ellipsis;
    -o-text-overflow: ellipsis;
}

#mailcontent {
    position: absolute;
    top: 0;
    left: 0;
    border: 0;
    margin: 0;
    padding: 10px;
    width: 100%;
}

.unseen {
    font-weight: bold;
}

#left-toolbar { 
    position: absolute;
    margin-left: 0px;
    width: 100%;
    left: 0;
    bottom: 0;
}

#quotabox {
    padding-top: 15px;
}

#quotabar {
    height: 5px;
    margin-bottom: 0;
}

#quotaraw {
    margin-left: 10px;
}


/*
 * Mailboxes list
 */
.email {
    border-top: 1px solid #ddd;
    padding-top: 10px;
}

.flag {
    cursor: pointer;
    margin-left: 10px;
}

div.email:hover { 
    background-color: #f5f5f5;
}

.openable {
    cursor: pointer;
}

.leftcol-heading { 
    margin-bottom: 15px;
}

#mboxactions {
    margin: 5px 0;
}

div.clickbox {
    position: absolute;
    left: 10px;
    top: 5px;
    width: 14px;
    height: 14px;
    z-index: 100;
}

div.collapsed {
    background-image: url("../../pics/collapsed.png");
    background-repeat: no-repeat;
}

div.expanded {
    background-image: url("../../pics/expanded.png");
    background-repeat: no-repeat;
}

#mboxes_container .nav .droppable div.clickbox {
    z-index: 800;
}

/*
 * Compose form
 */
#mailheader {
    position: relative;
    padding-bottom: 5px;
}

#mailheader label {
    text-align: left;
}

#mailheader .options {
    margin-bottom: 8px;
}

#attachments {
    position: relative;
    margin: 1px 0 5px 0;
    padding: 5px 0;
    cursor: pointer;
    width: 100%;
}

#attachments img {
    float: left;
}

list {
    position: absolute;
    top: 5px;
    left: 180px;
    font-weight: bold;
}

#body_container {
    position: static;
    /*height: 600px;*/
}

#id_body {
    width: 100%;
    height: 100%;
}

/*
 * Attachments form
 */
#submit {
    margin: 0 auto;
}

#attachment_list {
    margin-top: 20px;
    height: 150px;
    overflow-x: hidden;
    overflow-y: auto;
    padding: 2px;
}

#attachment_list i,
#attachment_list label {
    float: left;
}

/*
 * Drap & Drop
 */
.dragbox {
    padding : 5px;
    text-align : center;
    z-index : 1001;
}

/*
 * Folders form
 */

.modal-body {
    max-height: 300px;
    overflow-y: auto;
}

#folders2 li {
    position: relative;
}

#folders2 div.clickbox { 
    left: 0;
    top: 0px;
}

#folders2 li li { 
    margin-left: 10px;
}

#folders2 .active > a,
#folders2 .active > a:hover,
#folders2 .active > a:focus {
    color: #fff;
    background-color: #428bca;
}

/*
 * Table
 */



/*
 * Action buttons
 */

#menubar {
    padding-left: 15px;
    padding-bottom: 20px;
    z-index: 100;
}

#menubar .btn-toolbar ul.navbar-right{
    margin-right: 3%;
}

#menubar .btn-toolbar .navbar-form{
    margin: 0px;
}

#emailheaders {
    position: absolute;
    top: 0;
    left: 0;
}