{% extends "common/generic_modal_form.html" %}
{% load i18n %}
{% load static %}
{% block modalbody %}
{{ block.super }}

<div class="col-sm-12 text-center">
  <input type="button" class="btn btn-primary" id="submit" name="submit" value="{% trans 'Attach' %}" />
</div>

<div id="upload_status" style="display:none">
  <img src="{% static 'css/spinner.gif' %}" /> {% trans "Uploading.." %}
</div>
<iframe id="upload_target" name="upload_target" src="#" 
        style="width:0;height:0;border:0px solid #fff;"></iframe>
<div class="well" id="attachment_list">
  {% for att in attachments %}
  <div class="row">
    <div class="col-sm-12">
      <a href="{% url 'kalabash_webmail:attachment_delete' %}?name={{ att.tmpname }}"
         name="delattachment">
        <span class="fa fa-remove"></span>
      </a>
      <label>{{ att.fname }}</label>
    </div>
  </div>
  {% endfor %}
</div>
{% endblock %}
