#attachment_list {
    border: 1px solid #ccc;
    background-color: #eee;
    position: absolute;
    top: 140px;
    left: 20px;
    right: 20px;
    bottom: 35px;
    overflow-x: hidden;
    overflow-y: auto;
    padding: 2px;
}

#upload_status {
    position: relative;
    margin: 0 auto;
    font-size: 0.9em;
    width: 120px;
    font-style: italic;
}

#done_container {
    position: absolute;
    bottom: 0;
    width: 100%;
    text-align: center;
}

img {
    float: left;
    margin-right: 5px;
}

div.row label {
    width: 150px;
}

#id_attachment {
    width: 290px;
}
