{% extends "common/generic_modal_form.html" %}
{% load i18n webmail_tags %}
{% block extrafields %}
<div id="folders2" class="well">
  <ul class="nav nav-pills nav-stacked">
    <li class="nav-header">{% trans "Parent mailbox" %}</li>
    {% print_mailboxes mboxes selected withunseen selectonly hdelimiter %}
  </ul>
</div>
{% endblock %}
